package tools.descartes.dql.connector.simQPN.testUtils;

import static org.junit.Assert.fail;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.List;

import tools.descartes.dql.models.mapping.domain.Aggregate;
import tools.descartes.dql.models.mapping.domain.DecimalResult;
import tools.descartes.dql.models.mapping.domain.DoF;
import tools.descartes.dql.models.mapping.domain.Probe;
import tools.descartes.dql.models.mapping.domain.Resource;
import tools.descartes.dql.models.mapping.domain.Service;
import tools.descartes.dql.models.mapping.mapping.EntityMapping;

public class TestUtil {
	
	public static String listToString(List<String> list) {
		String ret = "";
		boolean firstline = true;
		for (String s: list) {
			if (!firstline)
				ret += "\n";
			else
				firstline = false;
			ret += s;
		}
		return ret;
	}
	
	public static Object[][] loadArrayFromFile(String path) {
		String content = TestUtil.loadFromFile(path);
		String[] lines = content.split("\n");
		Object[][] ret = new Object[lines.length][4];
		for (int i = 0; i < lines.length; i++) {
			String[] values = lines[i].split(" ");
			ret[i][0] = values[0];
			ret[i][1] = values[1];
			ret[i][2] = Double.valueOf(values[2]);
			ret[i][3] = Double.valueOf(values[3]);
		}
		return ret;
	}
	
	public static Double getValueOf(EntityMapping mapping, String resourceName, String metricName) {
		for (Resource r: mapping.getResources())
			if (r.getIdentifier().equals(resourceName))
				return getValueOf(r, metricName);
		throw new IllegalStateException("There is no resource with the name " + resourceName);
	}
	
	private static Double getValueOf(Resource r, String metricName) {
		for (Probe probe: r.getProbes())
			if (probe.getMetricName().equals(metricName) && probe instanceof DecimalResult) {
				DecimalResult dr = (DecimalResult)probe;
				return Double.valueOf(dr.getValue().toString());
			}
		return null;
	}

	public static String dumpResultsToString(EntityMapping em) {
		String result = "";
		for (Resource r: em.getResources())
			for (Probe p: r.getProbes())
				if (p instanceof DecimalResult) {
					DecimalResult dr = (DecimalResult)p;
					result += "\n" + r.getIdentifier() + "." + dr.getMetricName() 
							+ " " + dr.getValue();
				}
		return result;
	}
	
	public static String loadFromFile(String path) {
	    try {
			return new String(Files.readAllBytes(Paths.get(path)));
		} catch (IOException e) {
			e.printStackTrace();
			fail("Couldn't read file!");
			return "";
		}
	}
	
	public static void saveToFile(String path, String content) {
		try {
			Files.write( Paths.get(path), content.getBytes());
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public static boolean contentCompare(String s1, String s2) {
		s1 = s1.replaceAll("\\s", "");
		s2 = s2.replaceAll("\\s", "");
		return s1.equals(s2);
	}
	
	public static String entityMapptingToString(EntityMapping em) {
		String ret = "";
//		
//		//add ModelLocation
//		ret += em.getModelLocation() + "\n";
//		ret += "\n";
		
		//add DoF
		for (DoF dof: em.getDoF())
			ret += doFToString(dof) + "\n";
		ret += "\n";
		
		//add Aggreates
		for (Aggregate a: em.getAggregates())
			ret += aggregateToString(a) + "\n";
		ret += "\n";
		
		//add Resources
		for (Resource r: em.getResources())
			ret += resourceToString(r) + "\n";
		ret += "\n";
		
		//add Services
		for (Service s: em.getServices())
			ret += serviceToString(s) + "\n";
		ret += "\n";
		
		return ret;		
	}

	private static String serviceToString(Service s) {
		String ret = "";
		return ret;
	}

	private static String resourceToString(Resource r) {
		String ret = "";
		ret += r.getAlias() + "\n";
		ret += r.getIdentifier() + "\n";
		for (Probe p: r.getProbes())
			ret += "\t" + probeToString(p) + "\n";
		return ret;
	}

	private static String probeToString(Probe p) {
		if (p instanceof DecimalResult) {
			return decimalResultToString((DecimalResult)p);
		}
		String ret = "";
		ret += p.getMetricName();
		return ret;
	}

	private static String decimalResultToString(DecimalResult p) {
		String ret = "";
		ret += p.getMetricName();
		ret += " " + p.getAccuracy();
		ret += " " + p.getValid();
		ret += " " + Double.valueOf(p.getValue().toString());
		return ret;
	}

	private static String aggregateToString(Aggregate a) {
		return "";
	}

	private static String doFToString(DoF dof) {
		String ret = "";
		ret += dof.getAlias() + "\n";
		ret += dof.getIdentifier();
		return ret;
	}
}
